import os
import numpy as np
from pycollector.globals import print
from vipy.util import findpkl, toextension, filepath, filebase, jsonlist, ishtml, ispkl, filetail, temphtml, listpkl, listext, templike, tempdir, remkdir, tolist, fileext, writelist, tempcsv, newpathroot, listjson, extlist, filefull, tempdir
import random
import vipy
import vipy.util
import shutil
import uuid
import warnings
import copy 
import atexit
from pycollector.util import is_email_address
import torch
from vipy.batch import Batch       
import hashlib
import torch.utils.data
from torch.utils.data import DataLoader, random_split
import pickle
import time
import json
import dill
import vipy.torch
import vipy.dataset

    
def disjoint_activities(V, activitylist):    
    assert all([isinstance(v, vipy.video.Video) for v in V])
    assert isinstance(activitylist, list) and len(activitylist)>0 and len(activitylist[0]) == 2
    for (after, before) in activitylist:
        V = [v.activitymap(lambda a: a.disjoint([sa for sa in v.activitylist() if sa.category() == after]) if a.category() == before else a) for v in V]  
    V = [v.activityfilter(lambda a: len(a)>0) for v in V]  # some activities may be zero length after disjoint
    return V


def asmeva(V):
    """Convert a list of collector.dataset.Video() to MEVA annotation style"""
    assert all([isinstance(v, vipy.video.Scene) for v in V])
    
    # MEVA annotations assumptions:  https://docs.google.com/spreadsheets/d/19I3C5Zb6RHS0QC30nFT_m0ymArzjvlPLfb5SSRQYLUQ/edit#gid=0
    # Pad one second before, zero seconds after
    before1after0 = set(['person_opens_facility_door', 'person_closes_facility_door', 'person_opens_car_door', 'person_closes_car_door', 
                         'person_opens_car_trunk', 'person_opens_motorcycle_trunk', 'person_closes_car_trunk', 'person_closes_motorcycle_trunk',
                         'car_stops', 'motorcycle_stops', 'person_interacts_with_laptop'])        
    
    V = [v.activitymap(lambda a: a.temporalpad( (v.framerate()*1.0, 0) ) if a.category() in before1after0 else a) for v in V]
    
    # pad one second before, one second after, up to maximum of two seconds
    before1after1max2 = set(['person_enters_scene_through_structure'])
    V = [v.activitymap(lambda a: a.temporalpad(max(0, (v.framerate()*1.0)-len(a))) if a.category() in before1after1max2 else a) for v in V]
    
    # person_exits_scene_through_structure:  Pad one second before person_opens_facility_door label (if door collection), and ends with enough padding to make this minimum two seconds
    V = [v.activitymap(lambda a: (a.startframe(np.min([sa.startframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_opens_facility_door'] + [a.startframe()]))
                                  .temporalpad( (0, 0) )  # padding is rolled into person_opens_*
                                  if a.category() == 'person_exits_scene_through_structure' else a)) for v in V]        
    
    # person_enters_vehicle: Starts one second before person_opens_vehicle_door activity label and ends at the end of person_closes_vehicle_door activity, split motorcycles into separate class
    V = [v.activitymap(lambda a: (a.startframe(np.min([sa.startframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_opens_car_door'] + [a.startframe()]))
                                  .endframe(np.max([sa.endframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_closes_car_door'] + [a.endframe()]))
                                  .temporalpad( (0, 0) )  # padding is rolled into person_opens_*
                                  if a.category() == 'person_enters_car' else a)) for v in V]        
    
    # person_exits_vehicle:  Starts one second before person_opens_vehicle_door, and ends at person_exits_vehicle with enough padding to make this minimum two seconds, split motorcycles into separate class
    V = [v.activitymap(lambda a: (a.startframe(np.min([sa.startframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_opens_car_door'] + [a.startframe()]))
                                  .endframe(np.max([sa.endframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_closes_car_door'] + [a.endframe()]))
                                  .temporalpad( (0, 0) )  # padding is rolled into person_opens_*
                                  if a.category() == 'person_exits_car' else a)) for v in V]        
    
    # person_unloads_vehicle:  No padding before label start (the definition states one second of padding before cargo starts to move, but our label starts after the trunk is open, 
    # so there is a lag from opening to touching the cargo which we assume is at least 1sec), ends at the end of person_closes_trunk.
    V = [v.activitymap(lambda a: (a.endframe(np.max([sa.endframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_closes_car_trunk'] + [a.endframe()]))
                                  if a.category() == 'person_unloads_car' else a)) for v in V]
    V = [v.activitymap(lambda a: (a.endframe(np.max([sa.endframe() for sa in v.activities().values() if sa.isneighbor(a) and sa.category() == 'person_closes_motorcycle_trunk'] + [a.endframe()]))
                                  if a.category() == 'person_unloads_motorcycle' else a)) for v in V]
    
    # person_talks_to_person:  Equal padding to minimum of five seconds
    equal5 = set(['person_talks_to_person', 'person_reads_document'])
    V = [v.activitymap(lambda a: a.temporalpad(max(0, (v.framerate()*2.5)-len(a))) if a.category() in equal5 else a) for v in V]
    
    # person_texting_on_phone:  Equal padding to minimum of two seconds
    V = [v.activitymap(lambda a: a.temporalpad(max(0, (v.framerate()*1.0)-len(a))) if a.category() == 'person_texting_on_phone' else a) for v in V]
    
    # Pad one second before, one second after
    before1after1 = set(['car_turns_left', 'motorcycle_turns_left', 'car_turns_right', 'motorcycle_turns_right', 'person_transfers_object_to_person', 'person_transfers_object_to_vehicle',
                         'person_sets_down_object', 'hand_interacts_with_person_handshake', 'hand_interacts_with_person_highfive', 'hand_interacts_with_person_holdhands', 'person_embraces_person', 'person_purchases',
                         'car_picks_up_person', 'car_drops_off_person', 'motorcycle_drops_off_person', 'motorcycle_picks_up_person'])
    V = [v.activitymap(lambda a: a.temporalpad(v.framerate()*1.0) if a.category() in before1after1 else a) for v in V]
    
    # Pad zero second before, one second after
    before0after1 = set(['car_makes_u_turn', 'motorcycle_makes_u_turn', 'person_picks_up_object'])  
    V = [v.activitymap(lambda a: a.temporalpad( (0, v.framerate()*1.0) ) if a.category() in before0after1 else a) for v in V]
    
    # person_abandons_package:  two seconds before, two seconds after
    V = [v.activitymap(lambda a: a.temporalpad(v.framerate()*2.0) if a.category() == 'person_abandons_package' else a) for v in V]
    return V




class TorchDataset(vipy.torch.TorchDataset):
    """Moved to vipy.torch.TorchDataset"""
    # vipy-1.11.7    
    pass

class TorchTensordir(vipy.torch.TorchTensordir):
    """Moved to vipy.torch.TorchTensordir"""
    # vipy-1.11.7    
    pass

class Dataset(vipy.dataset.Dataset):
    """Additional methods beyond `vipy.dataset.Dataset` that are specific to collector style datasets"""

    def collectors(self, outfile=None):
        assert self._isvipy()
        d = vipy.util.countby(self.list(), lambda v: v.attributes['collector_id'])
        f = lambda x,n: len([k for (k,v) in d.items() if int(v) >= n])
        print('[vipy.dataset]: Collectors = %d ' % f(d,0))
        print('[vipy.dataset]: Collectors with >10 submissions = %d' % f(d,10))
        print('[vipy.dataset]: Collectors with >100 submissions = %d' % f(d,100))
        print('[vipy.dataset]: Collectors with >1000 submissions = %d' % f(d,1000))
        print('[vipy.dataset]: Collectors with >10000 submissions = %d' % f(d,10000))

        if outfile is not None:
            from vipy.metrics import histogram
            (k,v) = zip(*(sorted(d.items(), key=lambda x: x[1], reverse=True))) 
            histogram(v, list(range(len(k))), outfile=outfile, ylabel='Submissions', xlabel='Collector ID', xrot='vertical', fontsize=3, xshow=False)            
        return d

    def os(self, outfile=None):
        assert self._isvipy()
        d = vipy.util.countby([v for v in self.list() if v.hasattribute('device_identifier')], lambda v: v.attributes['device_identifier'])
        print('[vipy.dataset]: Device OS = %d ' % len(d))
        if outfile is not None:
            from vipy.metrics import pie
            pie(d.values(), d.keys(), explode=None, outfile=outfile,  shadow=False)
        return d

    def device(self, outfile=None, n=24, fontsize=7):
        assert self._isvipy()
        d_all = vipy.util.countby([v for v in self.list() if v.hasattribute('device_type') and v.attributes['device_type'] != 'unrecognized'], lambda v: v.attributes['device_type'])
        
        topk = [k for (k,v) in sorted(list(d_all.items()), key=lambda x: x[1])[-n:]] 
        other = np.sum([v for (k,v) in d_all.items() if k not in set(topk)])

        d = {k:v for (k,v) in d_all.items() if k in set(topk)}
        d.update( {'Other':other} )
        d = dict(sorted(list(d.items()), key=lambda x: x[1]))

        print('[vipy.dataset.device]: Device types = %d ' % len(d_all))
        print('[vipy.dataset.device]: Top-%d Device types = %s ' % (n, str(topk)))

        if outfile is not None:
            from vipy.metrics import pie
            pie(d.values(), d.keys(), explode=None, outfile=outfile,  shadow=False, legend=False, fontsize=fontsize, rotatelabels=False)
        return d
        



import heyvi
assert heyvi.version.is_at_least('0.0.3')


class ActivityRecognition(heyvi.recognition.ActivityRecognition):
    pass
    
class PIP_250k(heyvi.recognition.PIP_250k):
    pass

class PIP_370k(heyvi.recognition.PIP_370k):
    pass

class ActivityTracker(heyvi.recognition.ActivityTracker): 
    pass





class Label(object):
    def __init__(self, labeldict):
        pass

    
def _v1_applabel_to_piplabel(k=None):
    d = {"P002_P002C001_Closing": "person_closes_car_door",
         "P002_P002C001_Entering": "person_enters_car",
         "P002_P002C001_Exiting": "person_exits_car",
         "P002_P002C001_Opening": "person_opens_car_door",
         "P002_P002C002_Closing": "person_closes_car_trunk",
         "P002_P002C002_Loading": "person_loads_car",
         "P002_P002C002_Opening": "person_opens_car_trunk",
         "P002_P002C002_Unloading": "person_unloads_car",
         "P002_P002C003_Turning left": "car_turns_left",
         "P002_P002C003_Turning right": "car_turns_right",
         "P002_P002C003_U-turn": "car_makes_u_turn",
         "P002_P002C004_Picking up": "car_picks_up_person",
         "P002_P002C004_Starting": "car_starts",
         "P002_P002C004_Stopping": "car_stops",
         "P002_P002C005_Dropping off": "car_drops_off_person",
         "P002_P002C005_Starting": "car_starts",
         "P002_P002C005_Stopping": "car_stops",
         "P002_P002C006_Transferring": "person_transfers_object_to_car",
         "P002_P002C007_Reversing": "car_reverses",
         "P003_P003C001_Closing": "person_closes_motorcycle_door",  # remove me, meaningless
         "P003_P003C001_Entering": "person_enters_motorcycle",
         "P003_P003C001_Exiting": "person_exits_motorcycle",
         "P003_P003C001_Opening": "person_opens_motorcycle_door",  # remove me, meaningless
         "P003_P003C002_Closing": "person_closes_motorcycle_trunk",
         "P003_P003C002_Loading": "person_loads_motorcycle",
         "P003_P003C002_Opening": "person_opens_motorcycle_trunk",
         "P003_P003C002_Unloading": "person_unloads_motorcycle",
         "P003_P003C003_Turning left": "motorcycle_turns_left",
         "P003_P003C003_Turning right": "motorcycle_turns_right",
         "P003_P003C003_U-turn": "motorcycle_makes_u_turn",
         "P003_P003C004_Picking up": "motorcycle_picks_up_person",
         "P003_P003C004_Starting": "motorcycle_starts",
         "P003_P003C004_Stopping": "motorcycle_stops",
         "P003_P003C005_Dropping off": "motorcycle_drops_off_person",
         "P003_P003C005_Starting": "motorcycle_starts",
         "P003_P003C005_Stopping": "motorcycle_stops",
         "P003_P003C006_Transferring": "person_transfers_object_to_motorcycle",
         "P003_P003C007_Reversing": "motorcycle_reverses",
         "P004_P004C001_Closing": "person_closes_facility_door",
         "P004_P004C001_Entering": "person_enters_scene_through_structure",
         "P004_P004C001_Exiting": "person_exits_scene_through_structure",
         "P004_P004C001_Opening": "person_opens_facility_door",
         "P004_P004C002_Entering": "person_enters_scene_through_structure",
         "P004_P004C002_Exiting": "person_exits_scene_through_structure",
         "P004_P004C003_Reading": "person_reads_document",
         "P004_P004C003_Sitting": "person_sits_down",
         "P004_P004C003_Standing": "person_stands_up",
         "P004_P004C004_Dropping off": "person_puts_down_object",
         "P004_P004C004_Picking up": "person_picks_up_object",
         "P004_P004C005_Abandoning": "person_abandons_package",
         "P004_P004C005_Sets down": "person_puts_down_object",
         "P004_P004C006_Carrying": "person_carries_heavy_object",
         "P004_P004C007_Talking on": "person_talks_on_phone",
         "P004_P004C007_Texting on": "person_texts_on_phone",
         "P004_P004C008_Sitting": "person_sits_down",
         "P004_P004C008_Standing": "person_stands_up",
         "P004_P004C008_Using laptop": "person_interacts_with_laptop",
         "P004_P004C009_Purchasing": "person_purchases_from_machine",
         "P004_P004C010_Dropping off": "person_puts_down_object",
         "P004_P004C010_Picking up": "person_picks_up_object",
         "P004_P004C010_Riding": "person_rides_bicycle",
         "P005_P005C001_Talking to": "person_talks_to_person",
         "P005_P005C001_Touching hands": "hand_interacts_with_person_highfive",
         "P005_P005C002_Stealing": "person_steals_object",
         "P005_P005C003_Purchasing": "person_purchases_from_cashier",  # Remove me
         "P005_P005C004_Transferring": "person_transfers_object_to_person",
         "P005_P005C005_Hugging": "person_embraces_person",
         "P005_P005C005_Touching hands": "hand_interacts_with_person_holdhands",
         "P005_P005C006_Talking to": "person_talks_to_person",
         "P005_P005C006_Touching hands": "hand_interacts_with_person_shakehands",
    }
    return d[k] if k is not None else d


def piplabel_to_mevalabel():
    d = {v: v for (k, v) in _v1_applabel_to_piplabel().items()}

    d = {k: "hand_interacts_with_person" if "hand" in v else v for (k, v) in d.items()}
    d = {
        k: v.replace("car", "vehicle")
        if (("_car" in v or "car_" in v) and ("carries" not in v))
        else v
        for (k, v) in d.items()
    }
    d = {
        k: v.replace("motorcycle", "vehicle") if "motorcycle" in v else v
        for (k, v) in d.items()
    }
    d = {
        k: v.replace("vehicle_trunk", "trunk") if "vehicle_trunk" in v else v
        for (k, v) in d.items()
    }
    d["person_transfers_object_to_car"] = "person_transfers_object"
    d["person_transfers_object_to_motorcycle"] = "person_transfers_object"
    d["person_transfers_object_to_person"] = "person_transfers_object"
    d["person_purchases_from_machine"] = "person_purchases"
    d["person_purchases_from_cashier"] = "person_purchases"
    d = {
        k: v
        for (k, v) in d.items()
        if k
        not in set(["person_closes_motorcycle_door", "person_opens_motorcycle_door"])
    }
    return d


def mevalabel_to_index():

    return {
        "hand_interacts_with_person": 0,
        "person_abandons_package": 1,
        "person_carries_heavy_object": 2,
        "person_closes_facility_door": 3,
        "person_closes_trunk": 4,
        "person_closes_vehicle_door": 5,
        "person_embraces_person": 6,
        "person_enters_scene_through_structure": 7,
        "person_enters_vehicle": 8,
        "person_exits_scene_through_structure": 9,
        "person_exits_vehicle": 10,
        "person_interacts_with_laptop": 11,
        "person_loads_vehicle": 12,
        "person_opens_facility_door": 13,
        "person_opens_trunk": 14,
        "person_opens_vehicle_door": 15,
        "person_picks_up_object": 16,
        "person_purchases": 17,
        "person_puts_down_object": 18,
        "person_reads_document": 19,
        "person_rides_bicycle": 20,
        "person_sits_down": 21,
        "person_stands_up": 22,
        "person_steals_object": 23,
        "person_talks_on_phone": 24,
        "person_talks_to_person": 25,
        "person_texts_on_phone": 26,
        "person_transfers_object": 27,
        "person_unloads_vehicle": 28,
        "vehicle_drops_off_person": 29,
        "vehicle_makes_u_turn": 30,
        "vehicle_picks_up_person": 31,
        "vehicle_reverses": 32,
        "vehicle_starts": 33,
        "vehicle_stops": 34,
        "vehicle_turns_left": 35,
        "vehicle_turns_right": 36,
    }


def piplabel_to_index():
    return {
        "car_drops_off_person": 0,
        "car_makes_u_turn": 1,
        "car_picks_up_person": 2,
        "car_reverses": 3,
        "car_starts": 4,
        "car_stops": 5,
        "car_turns_left": 6,
        "car_turns_right": 7,
        "hand_interacts_with_person_highfive": 8,
        "hand_interacts_with_person_holdhands": 9,
        "hand_interacts_with_person_shakehands": 10,
        "motorcycle_drops_off_person": 11,
        "motorcycle_makes_u_turn": 12,
        "motorcycle_picks_up_person": 13,
        "motorcycle_reverses": 14,
        "motorcycle_starts": 15,
        "motorcycle_stops": 16,
        "motorcycle_turns_left": 17,
        "motorcycle_turns_right": 18,
        "person_abandons_package": 19,
        "person_carries_heavy_object": 20,
        "person_closes_car_door": 21,
        "person_closes_car_trunk": 22,
        "person_closes_facility_door": 23,
        "person_closes_motorcycle_trunk": 24,
        "person_embraces_person": 25,
        "person_enters_car": 26,
        "person_enters_scene_through_structure": 27,
        "person_exits_car": 28,
        "person_exits_scene_through_structure": 29,
        "person_interacts_with_laptop": 30,
        "person_loads_car": 31,
        "person_loads_motorcycle": 32,
        "person_opens_car_door": 33,
        "person_opens_car_trunk": 34,
        "person_opens_facility_door": 35,
        "person_opens_motorcycle_trunk": 36,
        "person_picks_up_object": 37,
        "person_purchases_from_machine": 38,
        "person_puts_down_object": 39,
        "person_reads_document": 40,
        "person_rides_bicycle": 41,
        "person_sits_down": 42,
        "person_stands_up": 43,
        "person_steals_object": 44,
        "person_talks_on_phone": 45,
        "person_talks_to_person": 46,
        "person_texts_on_phone": 47,
        "person_transfers_object_to_car": 48,
        "person_transfers_object_to_motorcycle": 49,
        "person_transfers_object_to_person": 50,
        "person_unloads_car": 51,
        "person_unloads_motorcycle": 52,
    }



def pip_250k_powerset():
    return {('car_drops_off_person',): 0,
            ('car_drops_off_person', 'car_starts'): 1,
            ('car_drops_off_person', 'car_starts', 'car_stops'): 2,
            ('car_drops_off_person', 'car_stops'): 3,
            ('car_makes_u_turn',): 4,
            ('car_makes_u_turn', 'car_turns_left'): 5,
            ('car_makes_u_turn', 'car_turns_left', 'car_turns_right'): 6,
            ('car_makes_u_turn', 'car_turns_right'): 7,
            ('car_picks_up_person',): 8,
            ('car_picks_up_person', 'car_starts'): 9,
            ('car_picks_up_person', 'car_starts', 'car_stops'): 10,
            ('car_picks_up_person', 'car_stops'): 11,
            ('car_reverses',): 12,
            ('car_reverses', 'car_turns_left'): 13,
            ('car_reverses', 'car_turns_right'): 14,
            ('car_starts',): 15,
            ('car_starts', 'car_stops'): 16,
            ('car_starts', 'car_turns_left'): 17,
            ('car_starts', 'car_turns_right'): 18,
            ('car_stops',): 19,
            ('car_stops', 'car_turns_left'): 20,
            ('car_stops', 'car_turns_right'): 21,
            ('car_turns_left',): 22,
            ('car_turns_left', 'car_turns_right'): 23,
            ('car_turns_right',): 24,
            ('hand_interacts_with_person_highfive',): 25,
            ('hand_interacts_with_person_highfive', 'person_talks_to_person'): 26,
            ('motorcycle_drops_off_person',): 27,
            ('motorcycle_drops_off_person', 'motorcycle_starts'): 28,
            ('motorcycle_drops_off_person', 'motorcycle_starts', 'motorcycle_stops'): 29,
            ('motorcycle_drops_off_person', 'motorcycle_stops'): 30,
            ('motorcycle_makes_u_turn',): 31,
            ('motorcycle_makes_u_turn', 'motorcycle_turns_left'): 32,
            ('motorcycle_makes_u_turn',
             'motorcycle_turns_left',
             'motorcycle_turns_right'): 33,
            ('motorcycle_makes_u_turn', 'motorcycle_turns_right'): 34,
            ('motorcycle_picks_up_person',): 35,
            ('motorcycle_picks_up_person', 'motorcycle_starts'): 36,
            ('motorcycle_picks_up_person', 'motorcycle_starts', 'motorcycle_stops'): 37,
            ('motorcycle_picks_up_person', 'motorcycle_stops'): 38,
            ('motorcycle_reverses',): 39,
            ('motorcycle_starts',): 40,
            ('motorcycle_starts', 'motorcycle_stops'): 41,
            ('motorcycle_stops',): 42,
            ('motorcycle_turns_left',): 43,
            ('motorcycle_turns_left', 'motorcycle_turns_right'): 44,
            ('motorcycle_turns_right',): 45,
            ('person',): 46,
            ('person_abandons_object',): 47,
            ('person_abandons_object', 'person_puts_down_object_on_floor'): 48,
            ('person_carries_heavy_object',): 49,
            ('person_closes_car_door',): 50,
            ('person_closes_car_door', 'person_enters_car'): 51,
            ('person_closes_car_door',
             'person_enters_car',
             'person_exits_car',
             'person_opens_car_door'): 52,
            ('person_closes_car_door', 'person_enters_car', 'person_opens_car_door'): 53,
            ('person_closes_car_door', 'person_exits_car'): 54,
            ('person_closes_car_door', 'person_exits_car', 'person_opens_car_door'): 55,
            ('person_closes_car_door', 'person_loads_car'): 56,
            ('person_closes_car_door', 'person_opens_car_door'): 57,
            ('person_closes_car_door', 'person_unloads_car'): 58,
            ('person_closes_car_trunk',): 59,
            ('person_closes_car_trunk', 'person_loads_car'): 60,
            ('person_closes_car_trunk', 'person_loads_car', 'person_opens_car_trunk'): 61,
            ('person_closes_car_trunk', 'person_opens_car_trunk'): 62,
            ('person_closes_car_trunk',
             'person_opens_car_trunk',
             'person_unloads_car'): 63,
            ('person_closes_car_trunk', 'person_unloads_car'): 64,
            ('person_closes_facility_door',): 65,
            ('person_closes_facility_door', 'person_enters_scene_through_structure'): 66,
            ('person_closes_facility_door',
             'person_enters_scene_through_structure',
             'person_exits_scene_through_structure',
             'person_opens_facility_door'): 67,
            ('person_closes_facility_door',
             'person_enters_scene_through_structure',
             'person_opens_facility_door'): 68,
            ('person_closes_facility_door',
             'person_enters_scene_through_structure',
             'person_opens_facility_door',
             'person_talks_on_phone'): 69,
            ('person_closes_facility_door',
             'person_enters_scene_through_structure',
             'person_talks_on_phone'): 70,
            ('person_closes_facility_door', 'person_exits_scene_through_structure'): 71,
            ('person_closes_facility_door',
             'person_exits_scene_through_structure',
             'person_opens_facility_door'): 72,
            ('person_closes_facility_door',
             'person_exits_scene_through_structure',
             'person_opens_facility_door',
             'person_talks_on_phone'): 73,
            ('person_closes_facility_door',
             'person_exits_scene_through_structure',
             'person_talks_on_phone'): 74,
            ('person_closes_facility_door', 'person_opens_facility_door'): 75,
            ('person_closes_facility_door',
             'person_opens_facility_door',
             'person_talks_on_phone'): 76,
            ('person_closes_facility_door', 'person_talks_on_phone'): 77,
            ('person_closes_motorcycle_trunk',): 78,
            ('person_closes_motorcycle_trunk', 'person_loads_motorcycle'): 79,
            ('person_closes_motorcycle_trunk',
             'person_loads_motorcycle',
             'person_opens_motorcycle_trunk'): 80,
            ('person_closes_motorcycle_trunk', 'person_opens_motorcycle_trunk'): 81,
            ('person_closes_motorcycle_trunk',
             'person_opens_motorcycle_trunk',
             'person_unloads_motorcycle'): 82,
            ('person_closes_motorcycle_trunk', 'person_unloads_motorcycle'): 83,
            ('person_embraces_person',): 84,
            ('person_embraces_person', 'person_holds_hand'): 85,
            ('person_embraces_person', 'person_talks_to_person'): 86,
            ('person_enters_car',): 87,
            ('person_enters_car', 'person_opens_car_door'): 88,
            ('person_enters_scene_through_structure',): 89,
            ('person_enters_scene_through_structure',
             'person_exits_scene_through_structure'): 90,
            ('person_enters_scene_through_structure',
             'person_exits_scene_through_structure',
             'person_opens_facility_door'): 91,
            ('person_enters_scene_through_structure', 'person_opens_facility_door'): 92,
            ('person_enters_scene_through_structure', 'person_talks_on_phone'): 93,
            ('person_exits_car',): 94,
            ('person_exits_car', 'person_opens_car_door'): 95,
            ('person_exits_scene_through_structure',): 96,
            ('person_exits_scene_through_structure', 'person_opens_facility_door'): 97,
            ('person_exits_scene_through_structure', 'person_talks_on_phone'): 98,
            ('person_holds_hand',): 99,
            ('person_holds_hand', 'person_talks_to_person'): 100,
            ('person_interacts_with_laptop',): 101,
            ('person_interacts_with_laptop', 'person_sits_down'): 102,
            ('person_interacts_with_laptop', 'person_stands_up'): 103,
            ('person_loads_car',): 104,
            ('person_loads_car', 'person_opens_car_trunk'): 105,
            ('person_loads_motorcycle',): 106,
            ('person_loads_motorcycle', 'person_opens_motorcycle_trunk'): 107,
            ('person_opens_car_door',): 108,
            ('person_opens_car_trunk',): 109,
            ('person_opens_facility_door',): 110,
            ('person_opens_facility_door', 'person_talks_on_phone'): 111,
            ('person_opens_motorcycle_trunk',): 112,
            ('person_opens_motorcycle_trunk', 'person_unloads_motorcycle'): 113,
            ('person_picks_up_object_from_floor',): 114,
            ('person_picks_up_object_from_shelf',): 115,
            ('person_picks_up_object_from_table',): 116,
            ('person_picks_up_object_from_table',
             'person_puts_down_object_on_table',
             'person_reads_document'): 117,
            ('person_picks_up_object_from_table', 'person_reads_document'): 118,
            ('person_purchases_from_cashier',): 119,
            ('person_purchases_from_machine',): 120,
            ('person_puts_down_object_on_floor',): 121,
            ('person_puts_down_object_on_shelf',): 122,
            ('person_puts_down_object_on_table',): 123,
            ('person_puts_down_object_on_table', 'person_reads_document'): 124,
            ('person_reads_document',): 125,
            ('person_reads_document', 'person_sits_down'): 126,
            ('person_reads_document', 'person_sits_down', 'person_stands_up'): 127,
            ('person_reads_document', 'person_stands_up'): 128,
            ('person_rides_bicycle',): 129,
            ('person_shakes_hand',): 130,
            ('person_shakes_hand', 'person_talks_to_person'): 131,
            ('person_sits_down',): 132,
            ('person_stands_up',): 133,
            ('person_steals_object_from_person',): 134,
            ('person_talks_on_phone',): 135,
            ('person_talks_to_person',): 136,
            ('person_texts_on_phone',): 137,
            ('person_transfers_object_to_car',): 138,
            ('person_transfers_object_to_motorcycle',): 139,
            ('person_transfers_object_to_person',): 140,
            ('person_unloads_car',): 141,
            ('person_unloads_motorcycle',): 142}




pip_to_meva = {'person_enters_motorcycle': 'person_enters_vehicle',
               'person_exits_motorcycle': 'person_exits_vehicle',
               'person_closes_motorcycle_trunk': 'person_closes_trunk',
               'person_loads_motorcycle': 'person_loads_vehicle',
               'person_opens_motorcycle_trunk': 'person_opens_trunk',
               'person_unloads_motorcycle': 'person_unloads_vehicle',
               'motorcycle_turns_left': 'vehicle_turns_left',
               'motorcycle_turns_right': 'vehicle_turns_right',
               'motorcycle_makes_u_turn': 'vehicle_makes_u_turn',
               'motorcycle_picks_up_person': 'vehicle_picks_up_person',
               'motorcycle_starts': 'vehicle_starts',
               'motorcycle_stops': 'vehicle_stops',
               'motorcycle_drops_off_person': 'vehicle_drops_off_person',
               'person_transfers_object_to_motorcycle': 'person_transfers_object',
               'motorcycle_reverses': 'vehicle_reverses',
               'person_closes_car_door': 'person_closes_vehicle_door',
               'person_enters_car': 'person_enters_vehicle',
               'person_exits_car': 'person_exits_vehicle',
               'person_opens_car_door': 'person_opens_vehicle_door',
               'person_closes_car_trunk': 'person_closes_trunk',
               'person_loads_car': 'person_loads_vehicle',
               'person_opens_car_trunk': 'person_opens_trunk',
               'person_unloads_car': 'person_unloads_vehicle',
               'car_turns_left': 'vehicle_turns_left',
               'car_turns_right': 'vehicle_turns_right',
               'car_makes_u_turn': 'vehicle_makes_u_turn',
               'car_picks_up_person': 'vehicle_picks_up_person',
               'car_starts': 'vehicle_starts',
               'car_stops': 'vehicle_stops',
               'car_drops_off_person': 'vehicle_drops_off_person',
               'person_transfers_object_to_car': 'person_transfers_object',
               'car_reverses': 'vehicle_reverses',
               'person_closes_facility_door': 'person_closes_facility_door',
               'person_enters_scene_through_structure': 'person_enters_scene_through_structure',
               'person_comes_into_scene_through_structure': 'person_enters_scene_through_structure',
               'person_exits_scene_through_structure': 'person_exits_scene_through_structure',
               'person_leaves_scene_through_structure': 'person_exits_scene_through_structure',
               'person_opens_facility_door': 'person_opens_facility_door',
               'person_reads_document': 'person_reads_document',
               'person_sits_down': 'person_sits_down',
               'person_stands_up': 'person_stands_up',
               'person_picks_up_object_from_table': 'person_picks_up_object',
               'person_picks_up_object_from_floor': 'person_picks_up_object',
               'person_picks_up_object_from_shelf': 'person_picks_up_object',
               'person_puts_down_object_on_table': 'person_puts_down_object',
               'person_puts_down_object_on_floor': 'person_puts_down_object',
               'person_puts_down_object_on_shelf': 'person_puts_down_object',
               'person_puts_down_object': 'person_puts_down_object',
               'person_picks_up_object': 'person_picks_up_object',               
               'person_abandons_object': 'person_abandons_package',
               'person_abandons_bag': 'person_abandons_package',               
               'person_abandons_package': 'person_abandons_package',
               'person_carries_heavy_object': 'person_carries_heavy_object',
               'person_talks_on_phone': 'person_talks_on_phone',
               'person_texts_on_phone': 'person_texts_on_phone',
               'person_interacts_with_laptop': 'person_interacts_with_laptop',
               'person_purchases_from_machine': 'person_purchases',
               'person_purchases_from_cashier': 'person_purchases',               
               'person_rides_bicycle': 'person_rides_bicycle',
               'person_talks_to_person': 'person_talks_to_person',
               'hand_interacts_with_person_highfive': 'hand_interacts_with_person',
               'person_steals_object': 'person_steals_object',
               'person_steals_object_from_person': 'person_steals_object',
               'person_transfers_object_to_person': 'person_transfers_object',
               'person_embraces_person': 'person_embraces_person',
               'hand_interacts_with_person_holdhands': 'hand_interacts_with_person',
               'hand_interacts_with_person_shakehands': 'hand_interacts_with_person',
               'person_shakes_hand': 'hand_interacts_with_person',
               'person_holds_hand': 'hand_interacts_with_person'}
    

pip_to_shortlabel = {'car_drops_off_person': 'drops-off person',
                     'car_makes_u_turn': 'u-turn',
                     'car_picks_up_person': 'picks-up person',
                     'car_reverses': 'reverses',
                     'car_starts': 'starts',
                     'car_stops': 'stops',
                     'car_turns_left': 'turns left',
                     'car_turns_right': 'turns right',
                     'hand_interacts_with_person_highfive': 'high five',
                     'hand_interacts_with_person_holdhands': 'hold hands',
                     'hand_interacts_with_person_shakehands': 'shake hands',
                     'motorcycle_drops_off_person': 'drops-off person',
                     'motorcycle_makes_u_turn': 'u-turn',
                     'motorcycle_picks_up_person': 'picks-up person',
                     'motorcycle_reverses': 'reverses',
                     'motorcycle_starts': 'starts',
                     'motorcycle_stops': 'stops',
                     'motorcycle_turns_left': 'turns left',
                     'motorcycle_turns_right': 'turns right',
                     'person_abandons_object': 'abandons',                     
                     'person_abandons_bag': 'abandons',
                     'person_abandons_package': 'abandons',
                     'person_carries_heavy_object': 'carries',
                     'person_closes_car_door': 'closes car door',
                     'person_closes_car_trunk': 'closes trunk',
                     'person_closes_facility_door': 'closes door',
                     'person_closes_motorcycle_trunk': 'closes trunk',
                     'person_comes_into_scene_through_structure': 'enters scene',
                     'person_embraces_person': 'hugs',
                     'person_enters_car': 'enters car',
                     'person_enters_motorcycle': 'enters motorcycle',
                     'person_enters_scene_through_structure': 'enters scene',
                     'person_exits_car': 'exits car',
                     'person_exits_motorcycle': 'exits motorcycle',
                     'person_exits_scene_through_structure': 'exits scene',
                     'person_holds_hand': 'hold hands',
                     'person_interacts_with_laptop': 'uses laptop',
                     'person_leaves_scene_through_structure': 'exits scene',
                     'person_loads_car': 'loads',
                     'person_loads_motorcycle': 'loads',
                     'person_opens_car_door': 'opens car door',
                     'person_opens_car_trunk': 'opens trunk',
                     'person_opens_facility_door': 'opens door',
                     'person_opens_motorcycle_trunk': 'opens trunk',
                     'person_picks_up_object': 'picks up',
                     'person_picks_up_object_from_floor': 'picks up',
                     'person_picks_up_object_from_shelf': 'picks up',
                     'person_picks_up_object_from_table': 'picks up',
                     'person_purchases_from_cashier': 'purchases (cashier)',
                     'person_purchases_from_machine': 'purchases (machine)',
                     'person_puts_down_object': 'puts down',
                     'person_puts_down_object_on_floor': 'puts down',
                     'person_puts_down_object_on_shelf': 'puts down',
                     'person_puts_down_object_on_table': 'puts down',
                     'person_reads_document': 'reads',
                     'person_rides_bicycle': 'rides',
                     'person_shakes_hand': 'shake hands',
                     'person_sits_down': 'sits',
                     'person_stands_up': 'stands',
                     'person_steals_object': 'steals',
                     'person_steals_object_from_person': 'steals',
                     'person_talks_on_phone': 'talks on phone',
                     'person_talks_to_person': 'talks to',
                     'person_texts_on_phone': 'texts',
                     'person_transfers_object_to_car': 'transfers',
                     'person_transfers_object_to_motorcycle': 'transfers',
                     'person_transfers_object_to_person': 'transfers',
                     'person_unloads_car': 'unloads',
                     'person_unloads_motorcycle': 'unloads',
                     'person_walks': 'walks',
                     'person':'person',
                     'vehicle':'vehicle',
                     'car_moves':'moves'}


pip_plus_meva_to_meva = {k:v for (k,v) in list(pip_to_meva.items()) + [(k,k) for k in set(list(pip_to_meva.values()))]}




import os
import pytest
from datetime import datetime, timedelta
import pandas as pd
from pytz import timezone
import json

# internal testing modules
# from pycollector.user import User
from pycollector.project import Project

import pycollector


def _test_get_project_visym(org='visym', env='prod'):
    """test get project by user"""

    # Set to target ENV
    if org != 'visym' or env != 'prod':
        # Non-default backend requires admin tools
        from pycollector.admin.globals import backend    
        pycollector.admin.globals.backend(env="prod", org="visym")

    # testing variiables
    username = os.environ["VISYM_COLLECTOR_PYTEST_EMAIL"]  # github secrets
    password = os.environ["VISYM_COLLECTOR_PYTEST_PASSWORD"]  # github secrets

    #username = "heng2j@gmail.com"  # "zhongheng.li@stresearch.com"
    #password = None  # sanitized

    # testing objects and functions
    new_project_client = Project(program_id="Practice", alltime=True, username=username, password=password)

    # Fetching Videos
    videos = new_project_client.videos()
    assert len(videos) >= 1

    # quickshow video
    videos[0].show()



import os
import sys
import vipy
import numpy as np
import heyvi.detection
assert heyvi.version.is_at_least('0.0.3')


class TorchNet(heyvi.detection.TorchNet):
    pass

class FaceDetector(heyvi.detection.FaceDetector):
    pass
    
class Yolov5(heyvi.detection.Yolov5):
    pass
    
class Yolov3(heyvi.detection.Yolov3):
    pass
    
class ObjectDetector(heyvi.detection.ObjectDetector):
    pass

class MultiscaleObjectDetector(heyvi.detection.MultiscaleObjectDetector):  
    pass
    
class VideoDetector(heyvi.detection.VideoDetector):  
    pass
                       
class MultiscaleVideoDetector(heyvi.detection.MultiscaleVideoDetector):
    pass

class VideoTracker(heyvi.detection.VideoTracker):
    pass

class FaceTracker(heyvi.detection.FaceTracker):
    pass
    
class MultiscaleVideoTracker(heyvi.detection.MultiscaleVideoTracker):
    pass

class Proposal(heyvi.detection.Proposal):
    pass
    
class VideoProposal(heyvi.detection.VideoProposal):
    pass

class FaceProposalRefinement(heyvi.detection.FaceProposalRefinement):
    pass

class TrackProposalRefinement(heyvi.detection.TrackProposalRefinement):
    pass
    
class VideoProposalRefinement(heyvi.detection.VideoProposalRefinement):
    pass

class ActorAssociation(heyvi.detection.ActorAssociation):
    pass

